@@ -35,8 +35,7 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libgnomeui/gnome-icon-theme.h>
-#include <libgnomeui/gnome-icon-lookup.h>
+#include "gtk-utils.h"
 #include "quick-button.h"
 
 #define DEFAULT_OFFSET      2
@@ -74,7 +73,7 @@ static gboolean quick_button_leave_notify    (GtkWidget         *widget,
 					      GdkEventCrossing  *event);
 static void     quick_button_button_pressed  (GtkButton         *button);
 static void     quick_button_button_released (GtkButton         *button);
-void            set_pixmap                   (QuickButton       *button,  
+static void     set_pixmap                   (QuickButton       *button,  
 					      const char        *icon_path, 
 					      int                size);
 
@@ -208,14 +207,14 @@ quick_button_destroy (GtkObject *obj)
 		button->scaled_bright = NULL;
 	}
 	
-	if (button->icon_path != NULL) {
-		g_free (button->icon_path);
-		button->icon_path = NULL;
+	if (button->icon != NULL) {
+		g_free (button->icon);
+		button->icon = NULL;
 	}
 	
-	if (button->text != NULL) {
-		g_free (button->text);
-		button->text = NULL;
+	if (button->name != NULL) {
+		g_free (button->name);
+		button->name = NULL;
 	}
 
 	if (button->icon_theme != NULL) {
@@ -228,59 +227,6 @@ quick_button_destroy (GtkObject *obj)
 }
 
 
-GdkPixbuf *
-missing_pixbuf (int size)
-{
-        int        i, j;
-        int        rowstride;
-        guchar    *pixels;
-        GdkPixbuf *pb;
-
-        pb = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-                             FALSE /* has alpha */,
-                             8 /* bits per sample */,
-                             size, size);
-        pixels = gdk_pixbuf_get_pixels (pb);
-        rowstride = gdk_pixbuf_get_rowstride (pb);
-
-        for (i = 0; i < size; i++) {
-                for (j = 0; j < size; j++) {
-                        guchar *p = pixels + i * rowstride + 3 * j;
-                        if (i == j || size - i == j) {
-                                p[0] = 255;
-                                p[1] = 0;
-                                p[2] = 0;
-                        } else {
-                                p[0] = 255;
-                                p[1] = 255;
-                                p[2] = 255;
-                        }
-                }
-        }
-
-	return pb;
-}
- 
-
-static GdkPixbuf *
-get_missing (GnomeIconTheme *theme, 
-	     int             preffered_size)
-{
-	char *filename = NULL;
-
-	filename = gnome_icon_theme_lookup_icon (theme,
-						 "gnome-unknown",
-						 preffered_size,
-						 NULL,
-						 NULL);
-
-	if (filename == NULL)
-		return missing_pixbuf (preffered_size);
-		
-	return gdk_pixbuf_new_from_file (filename, NULL);
-}
-
-
 static void
 do_colorshift (GdkPixbuf *dest, 
 	       GdkPixbuf *src, 
@@ -345,28 +291,19 @@ make_bright_pixbuf (GdkPixbuf *pb)
 
 static void
 button_load_pixbuf (QuickButton *button,
-		    const char  *path,
+		    const char  *icon_name,
 		    int          preffered_size)
 
 {
-	GdkPixbuf       *new_pixbuf = NULL;
-	GError          *gerror = NULL;
-	int              w, h;
-	int              pixbuf_w, pixbuf_h;
-	double           scale;
+	GdkPixbuf *new_pixbuf = NULL;
+	int        w, h;
+	int        pixbuf_w, pixbuf_h;
+	double     scale;
 
 	if (preffered_size <= 0)
 		preffered_size = DEFAULT_SIZE;
-	
-	if (path == NULL) 
-		new_pixbuf = get_missing (button->icon_theme, preffered_size);
-	else
-		new_pixbuf = gdk_pixbuf_new_from_file (path, &gerror);
 
-	if (new_pixbuf == NULL) {
-		g_clear_error (&gerror);
-		new_pixbuf = get_missing (button->icon_theme, preffered_size);
-	}
+	new_pixbuf = create_pixbuf_or_missing (button->icon_theme, icon_name, preffered_size);	
 
 	/**/
 
@@ -382,16 +319,14 @@ button_load_pixbuf (QuickButton *button,
 	w = MAX ((double) w * scale, 1);
 	h = MAX ((double) h * scale, 1);
 
-	/**/
-
 	if (button->scaled != NULL)
 		g_object_unref (button->scaled);
-	if (button->scaled_bright != NULL)
-		g_object_unref (button->scaled_bright);
-
 	button->scaled = gdk_pixbuf_scale_simple (button->pixbuf, 
 						  w, h,
 						  GDK_INTERP_BILINEAR);
+						  
+	if (button->scaled_bright != NULL)
+		g_object_unref (button->scaled_bright);
 	button->scaled_bright = make_bright_pixbuf (button->scaled);
 }
 
@@ -444,16 +379,13 @@ quick_button_expose (GtkWidget         *widget,
 	area = event->area;
 	
 	if (gdk_rectangle_intersect (&area, &widget->allocation, &area) &&
-	    gdk_rectangle_intersect (&image_bound, &area, &image_bound))  
-		gdk_pixbuf_render_to_drawable_alpha (pixbuf,
-						     widget->window,
-						     image_bound.x - x, image_bound.y - y,
-						     image_bound.x, image_bound.y,
-						     image_bound.width, image_bound.height,
-						     GDK_PIXBUF_ALPHA_FULL,
-						     128,
-						     GDK_RGB_DITHER_NORMAL,
-						     0, 0);
+	    gdk_rectangle_intersect (&image_bound, &area, &image_bound))
+		gdk_draw_pixbuf (widget->window, NULL, pixbuf, 
+				 image_bound.x - x, image_bound.y - y,
+				 image_bound.x, image_bound.y,
+				 image_bound.width, image_bound.height,
+				 GDK_RGB_DITHER_NORMAL,
+				 0, 0);
 	
 	if (quick_button->dnd_highlight) 
 		gdk_draw_rectangle (widget->window, 
@@ -654,16 +586,16 @@ drag_data_get  (GtkWidget        *widget,
         gtk_selection_data_set (selection_data, 
                                 selection_data->target,
                                 8, 
-				uri,
+				(guchar*)uri,
                                 strlen (uri));
 }
 
 
 GtkWidget *
-quick_button_new (const char     *icon_path,
+quick_button_new (const char     *icon,
 		  int             size,
-		  const char     *text,
-		  GnomeIconTheme *icon_theme)
+		  const char     *name,
+		  GtkIconTheme   *icon_theme)
 {
 	QuickButton *button;
 
@@ -671,12 +603,12 @@ quick_button_new (const char     *icon_path,
 
 	button = QUICK_BUTTON (g_object_new (quick_button_get_type (), NULL));
 
-	button->icon_path = (icon_path != NULL) ? g_strdup (icon_path) : NULL;
+	button->icon = (icon != NULL) ? g_strdup (icon) : NULL;
 	button->size = size;
-	button->text = (text != NULL) ? g_strdup (text) : NULL;
+	button->name = (name != NULL) ? g_strdup (name) : NULL;
 	button->icon_theme = g_object_ref (icon_theme);
 
-	button_load_pixbuf (button, icon_path, size);
+	button_load_pixbuf (button, icon, size);
 
 	gtk_drag_source_set (GTK_WIDGET (button),
                              GDK_BUTTON1_MASK,
@@ -698,41 +630,40 @@ quick_button_new (const char     *icon_path,
 
 void
 quick_button_set_pixmap (QuickButton *button,
-			 const char  *icon_path,
+			 const char  *icon,
 			 int          size)
 {
-	QUICK_BUTTON_GET_CLASS (G_OBJECT (button))->set_pixmap (button, icon_path, size);
+	QUICK_BUTTON_GET_CLASS (G_OBJECT (button))->set_pixmap (button, icon, size);
 }
 
 
-void
+static void
 set_pixmap (QuickButton *button,
-	    const char  *icon_path,
+	    const char  *icon,
 	    int          size)
 {
 	g_return_if_fail (QUICK_IS_BUTTON (button));
 	g_return_if_fail (size > 0);
 
-	button->size = size;
-	if (icon_path != button->icon_path) {
-		g_free (button->icon_path);
-		button->icon_path = g_strdup (icon_path);
+	if (icon != button->icon) {
+		g_free (button->icon);
+		button->icon = g_strdup (icon);
 	}
-
-	button_load_pixbuf (button, icon_path, size);
-
+	button->size = size;
+	
+	button_load_pixbuf (button, icon, size);
 	gtk_widget_queue_resize (GTK_WIDGET (button));
 }
 
 
 void
 quick_button_set_text (QuickButton *button, 
-		       const char  *text)
+		       const char  *name)
 {
 	g_return_if_fail (QUICK_IS_BUTTON (button));
 
-	g_free (button->text);
-	button->text = text ? g_strdup (text ) : NULL;
+	g_free (button->name);
+	button->name = name ? g_strdup (name) : NULL;
 
 	gtk_widget_queue_draw (GTK_WIDGET (button));
 }
